package java.security;

import java.util.regex.Pattern;
import java.util.regex.Matcher;
import sun.security.jca.GetInstance;
import java.util.Iterator;
import sun.security.provider.SunEntries;
import sun.security.jca.Providers;
import sun.security.util.Debug;
import java.util.Random;

public class SecureRandom extends Random
{
    private static final Debug pdebug;
    private static final boolean skipDebug;
    private Provider provider;
    private SecureRandomSpi secureRandomSpi;
    private String algorithm;
    private static volatile SecureRandom seedGenerator;
    static final long serialVersionUID = 4940670005562187L;
    private byte[] state;
    private MessageDigest digest;
    private byte[] randomBytes;
    private int randomBytesUsed;
    private long counter;
    
    public SecureRandom() {
        super(0L);
        this.provider = null;
        this.secureRandomSpi = null;
        this.digest = null;
        this.getDefaultPRNG(false, (byte[])null);
    }
    
    public SecureRandom(byte[] array) {
        super(0L);
        this.provider = null;
        this.secureRandomSpi = null;
        this.digest = null;
        this.getDefaultPRNG(true, array);
    }
    
    private void getDefaultPRNG(boolean b, byte[] array) {
        Provider.Service service = null;
        String algorithm = null;
        for (Provider provider : Providers.getProviderList().providers()) {
            if (provider.getName().equals("SUN")) {
                algorithm = SunEntries.DEF_SECURE_RANDOM_ALGO;
                service = provider.getService("SecureRandom", algorithm);
                break;
            }
            service = provider.getDefaultSecureRandomService();
            if (service != null) {
                algorithm = service.getAlgorithm();
                break;
            }
            continue;
        }
        if (service == null) {
            algorithm = "SHA1PRNG";
            this.secureRandomSpi = new sun.security.provider.SecureRandom();
            this.provider = Providers.getSunProvider();
        }
        else {
            try {
                this.secureRandomSpi = (SecureRandomSpi)service.newInstance(null);
                this.provider = service.getProvider();
            }
            catch (NoSuchAlgorithmException cause) {
                throw new RuntimeException(cause);
            }
        }
        if (b) {
            this.secureRandomSpi.engineSetSeed(array);
        }
        if (this.getClass() == SecureRandom.class) {
            this.algorithm = algorithm;
        }
    }
    
    protected SecureRandom(SecureRandomSpi secureRandomSpi, Provider provider) {
        this(secureRandomSpi, provider, null);
    }
    
    private SecureRandom(SecureRandomSpi secureRandomSpi, Provider provider, String s) {
        super(0L);
        this.provider = null;
        this.secureRandomSpi = null;
        this.digest = null;
        this.secureRandomSpi = secureRandomSpi;
        this.provider = provider;
        this.algorithm = s;
        if (!SecureRandom.skipDebug && SecureRandom.pdebug != null) {
            SecureRandom.pdebug.println("SecureRandom." + s + " algorithm from: " + this.provider.getName());
        }
    }
    
    public static SecureRandom getInstance(String algorithm) throws NoSuchAlgorithmException {
        GetInstance.Instance instance = GetInstance.getInstance("SecureRandom", SecureRandomSpi.class, algorithm);
        return new SecureRandom((SecureRandomSpi)instance.impl, instance.provider, algorithm);
    }
    
    public static SecureRandom getInstance(String algorithm, String provider) throws NoSuchAlgorithmException, NoSuchProviderException {
        GetInstance.Instance instance = GetInstance.getInstance("SecureRandom", SecureRandomSpi.class, algorithm, provider);
        return new SecureRandom((SecureRandomSpi)instance.impl, instance.provider, algorithm);
    }
    
    public static SecureRandom getInstance(String algorithm, Provider provider) throws NoSuchAlgorithmException {
        GetInstance.Instance instance = GetInstance.getInstance("SecureRandom", SecureRandomSpi.class, algorithm, provider);
        return new SecureRandom((SecureRandomSpi)instance.impl, instance.provider, algorithm);
    }
    
    public final Provider getProvider() {
        return this.provider;
    }
    
    public String getAlgorithm() {
        return (this.algorithm != null) ? this.algorithm : "unknown";
    }
    
    public synchronized void setSeed(byte[] array) {
        this.secureRandomSpi.engineSetSeed(array);
    }
    
    @Override
    public void setSeed(long n) {
        if (n != 0L) {
            this.secureRandomSpi.engineSetSeed(longToByteArray(n));
        }
    }
    
    @Override
    public void nextBytes(byte[] array) {
        this.secureRandomSpi.engineNextBytes(array);
    }
    
    @Override
    protected final int next(int n) {
        int n2 = (n + 7) / 8;
        byte[] array = new byte[n2];
        int n3 = 0;
        this.nextBytes(array);
        for (int i = 0; i < n2; ++i) {
            n3 = (n3 << 8) + (array[i] & 0xFF);
        }
        return n3 >>> n2 * 8 - n;
    }
    
    public static byte[] getSeed(int n) {
        if (SecureRandom.seedGenerator == null) {
            SecureRandom.seedGenerator = new SecureRandom();
        }
        return SecureRandom.seedGenerator.generateSeed(n);
    }
    
    public byte[] generateSeed(int n) {
        if (n < 0) {
            throw new NegativeArraySizeException("numBytes cannot be negative");
        }
        return this.secureRandomSpi.engineGenerateSeed(n);
    }
    
    private static byte[] longToByteArray(long n) {
        byte[] array = new byte[8];
        for (int i = 0; i < 8; ++i) {
            array[i] = (byte)(int)n;
            n >>= 8;
        }
        return array;
    }
    
    public static SecureRandom getInstanceStrong() throws NoSuchAlgorithmException {
        String group = (String)AccessController.doPrivileged(new PrivilegedAction<String>() {
            @Override
            public String run() {
                return Security.getProperty("securerandom.strongAlgorithms");
            }
        });
        if (group == null || group.length() == 0) {
            throw new NoSuchAlgorithmException("Null/empty securerandom.strongAlgorithms Security Property");
        }
        while (group != null) {
            Matcher matcher;
            if ((matcher = SecureRandom.StrongPatternHolder.pattern.matcher(group)).matches()) {
                String group2 = matcher.group(1);
                String group3 = matcher.group(3);
                try {
                    if (group3 == null) {
                        return getInstance(group2);
                    }
                    return getInstance(group2, group3);
                }
                catch (NoSuchAlgorithmException | NoSuchProviderException ex) {
                    group = matcher.group(5);
                }
            }
            else {
                group = null;
            }
        }
        throw new NoSuchAlgorithmException("No strong SecureRandom impls available: " + group);
    }
    
    static {
        pdebug = Debug.getInstance("provider", "Provider");
        skipDebug = (Debug.isOn("engine=") && !Debug.isOn("securerandom"));
        SecureRandom.seedGenerator = null;
    }
    
    private static final class StrongPatternHolder
    {
        private static Pattern pattern;
        
        static {
            SecureRandom.StrongPatternHolder.pattern = Pattern.compile("\\s*([\\S&&[^:,]]*)(\\:([\\S&&[^,]]*))?\\s*(\\,(.*))?");
        }
    }
}
